﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.IO;

namespace StickyNotes
{
    public partial class Form1 : Form
    {
        StickyNotes notes = new StickyNotes();
        public String file_name;

        // nastavení parametrů okna
        public Form1(String file_name, int width, int height, int x, int y, int red, int green, int blue, String text)
        {
            InitializeComponent();

            // ošetření vyskočení okna mimo obrazovku
            if ((x >= Screen.PrimaryScreen.Bounds.Width - 5) || (x < 0 - width + 5)) x = Screen.PrimaryScreen.Bounds.Width / 2;
            if ((y >= Screen.PrimaryScreen.Bounds.Height -5) || (y < 0 - height +5)) y = Screen.PrimaryScreen.Bounds.Height / 2;
            
            // ošetření min. velikosti
            if (width < 250) width = 250;
            if (height < 150) height = 150;

            this.file_name = file_name;
            this.Width = width;
            this.Height = height;
            this.Left = x;
            this.Top = y;
            notes.setColors(this, red, green, blue);

            setComponentProperties();
            this.richTextBox1.Text = text;

            notes.saveFiles(this);

            setButtonVisible(false);
        }

        // nastavení velikosti a pozice komponent
        private void setComponentProperties()
        {
            this.richTextBox1.Size = new Size(this.Width - this.richTextBox1.Location.X, this.Height - this.richTextBox1.Location.Y);
            this.richTextBox1.SelectionIndent = 5;

            this.close_button.Left = this.Width - close_button.Width - 3;

            this.label1.Left = this.Width - this.label1.Width;
            this.label1.Top = this.Height - this.label1.Height;
        }

        private void setButtonVisible(bool visible)
        {
            this.close_button.Visible = visible;
            this.new_note_button.Visible = visible;
        }

        private void close_button_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Opravdu chceš smazat poznámku?", "StickyNotes", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == System.Windows.Forms.DialogResult.Yes)
            {
                notes.deleteNote(file_name);
                this.Close();
            }
        }

        private void new_note_button_Click(object sender, EventArgs e)
        {
            notes.createNew(this.Left - 255, this.Top, this.richTextBox1.BackColor.R, this.richTextBox1.BackColor.G, this.richTextBox1.BackColor.B);
        }

        // spouštění odkazů z richtextboxu
        private void richTextBox1_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            Process.Start(e.LinkText);
        }

        #region Context menu richTextBoxu
        private void vyjmoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(richTextBox1.SelectedText, TextDataFormat.Text);
            richTextBox1.SelectedText = "";
        }

        private void kopírovatToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(richTextBox1.SelectedText, TextDataFormat.Text);
        }

        private void vložitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            richTextBox1.SelectedText = Clipboard.GetText(TextDataFormat.Text);
        }

        private void smazatToolStripMenuItem_Click(object sender, EventArgs e)
        {
            richTextBox1.SelectedText = "";
        }

        private void vybratVšeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            richTextBox1.SelectAll();
        }

        private void modráToolStripMenuItem_Click(object sender, EventArgs e)
        {
            notes.setColors(this, 200, 230, 255);
        }

        private void žlutáToolStripMenuItem_Click(object sender, EventArgs e)
        {
            notes.setColors(this, 255, 255, 128);
        }

        private void zelenáToolStripMenuItem_Click(object sender, EventArgs e)
        {
            notes.setColors(this, 200, 255, 200);
        }

        private void růžováToolStripMenuItem_Click(object sender, EventArgs e)
        {
            notes.setColors(this, 255, 200, 255);
        }

        private void fialováToolStripMenuItem_Click(object sender, EventArgs e)
        {
            notes.setColors(this, 200, 200, 255);
        }

        private void bíláToolStripMenuItem_Click(object sender, EventArgs e)
        {
            notes.setColors(this, 245, 245, 245);
        }
        #endregion

        // při zavření poznámky -> kontrola počtu poznámek (pokud byla zavřena poslední vypne se aplikaceú
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            notes.checkNotesCount();
        }
        
        // resizování poznámky
        private void label1_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                int width = (MousePosition.X - this.Left + 5) >= 250 ? MousePosition.X - this.Left + 5 : 250;
                int height = (MousePosition.Y - this.Top + 5) >= 150 ? MousePosition.Y - this.Top + 5 : 150;
                this.Size = new Size(width, height);
                setComponentProperties();
            }
        }

        private void Form1_Activated(object sender, EventArgs e)
        {
            setButtonVisible(true);
        }

        private void Form1_Deactivate(object sender, EventArgs e)
        {
            setButtonVisible(false);

            if (File.Exists(notes.user_appdata + "\\" + this.file_name))
                notes.saveFiles(this);
        }

        private void Form1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Middle)
                MessageBox.Show("StickyNotes\nJednoduché poznámky na plochu kompatibilní s Windows XP, Vista, 7. Vytvořené právě z důvodu postradatelnosti na starších systémech.\nVytvořil sczdavos [David Jančík] (www.islandsoft.cz)\nAutor nezodpovídá za škody způsobené programem.", "StickyNotes - EsterEgg", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // pohybování s oknem
        public const int WM_NCLBUTTONDOWN = 0xA1;
        public const int HTCAPTION = 0x2;
        [DllImportAttribute("user32.dll")]
        public static extern bool ReleaseCapture();
        [DllImportAttribute("user32.dll")]
        public static extern int SendMessage(IntPtr hWnd, int Msg, int wParam, int lParam);
        private void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            setButtonVisible(true);
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ReleaseCapture();
                SendMessage(Handle, WM_NCLBUTTONDOWN, HTCAPTION, 0);
            }
        }

        private void Form1_MouseLeave(object sender, EventArgs e)
        {
            if (ActiveForm != this)
                setButtonVisible(false);

            if (this.ClientRectangle.Contains(this.PointToClient(Control.MousePosition)))
            {
                setButtonVisible(true);
            }
        }
    }
}
