﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Drawing;
using Microsoft.Win32;
using System.Runtime.InteropServices;

namespace StickyNotes
{
    class StickyNotes
    {
        // cesta do user application data složky stickynotes
        public String user_appdata = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\StickyNotes";

        // najde všechny uložené poznámky a vytvoří dle nich instance
        public void openFiles()
        {
            try
            {
                // získá pole všech souborů uložených ve složce
                String[] user_notes = Directory.GetFiles(user_appdata);
                String[] note_properties;
                String text;

                foreach (String file in user_notes)
                {
                    text = "";
                    // načte konkrétní soubor -> file.opentext zachovává původní kódování
                    using (StreamReader sr = File.OpenText(file))
                    {
                        // načte do pole první řádek s nastavením
                        note_properties = sr.ReadLine().Split('|');

                        //zbytek nahraje do proměnné text případné \n odřádkuje
                        while (!sr.EndOfStream)
                            text += sr.ReadLine() + Environment.NewLine;
                    }
                    // vytvoří instanci poznámky
                    new Form1(Path.GetFileName(file), int.Parse(note_properties[0]), int.Parse(note_properties[1]), int.Parse(note_properties[2]), int.Parse(note_properties[3]), int.Parse(note_properties[4]), int.Parse(note_properties[5]), int.Parse(note_properties[6]), text).Show();
                }

                // pokud se ve složce nenachází žádné soubory vytvoří novou poznámku
                if (user_notes.Length == 0)
                    this.createNew(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 175, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 75, 255, 255, 128);
            }
            catch
            {
                this.createNew(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 175, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 75, 255, 255, 128);
            }
        }

        // uložení vlastností okna a poznámky pro zadanou instanci poznámky
        public void saveFiles(Form1 form)
        { 
            using (StreamWriter sw = new StreamWriter(new FileStream(user_appdata + "\\" + form.file_name, FileMode.Create)))
            {
                sw.WriteLine(form.Width + "|" + form.Height + "|" + form.Left + "|" + form.Top + "|" + ((int)form.BackColor.R + 10) + "|" + ((int)form.BackColor.G + 10) + "|" + ((int)form.BackColor.B + 10));
                sw.Write(form.richTextBox1.Text);
                sw.Flush();
            }
        }

        // vytvoří novou poznámku -> ukládá se jako timestamp
        public void createNew(int x, int y, int red, int green, int blue)
        {
            new Form1((DateTime.UtcNow.Ticks - DateTime.Parse("01/01/1970 00:00:00").Ticks).ToString(), 250, 150, x, y, red, green, blue, "").Show();
        }

        public void deleteNote(String file)
        {
            File.Delete(user_appdata + "\\" + file);
        }

        public void setColors(Form1 form, int red, int green, int blue)
        {
            form.BackColor = Color.FromArgb(red - 10, green - 10, blue - 10);
            form.richTextBox1.BackColor = Color.FromArgb(red, green, blue);

            form.close_button.FlatAppearance.BorderColor = form.BackColor;
            form.new_note_button.FlatAppearance.BorderColor = form.BackColor;
            form.label1.BackColor = form.richTextBox1.BackColor;
        }

        // kontroluje počet poznámek --> pokud byla zavřena poslední ukončí aplikaci
        public void checkNotesCount()
        {
            FormCollection forms = Application.OpenForms;
            if (forms.Count <= 1)
            {
                RegistryKey rkApp = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", true);
                rkApp.DeleteValue("StickyNotes", false);

                Application.Exit();
            }
        }

        // zkontroluje existenci složky a zápis v registrech po spuštění
        public void firstRun()
        {
            if (!Directory.Exists(user_appdata))
                Directory.CreateDirectory(user_appdata);

            RegistryKey rkApp = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", true);
            if (rkApp.GetValue("StickyNotes") == null)
            {
                try
                {
                    if (!File.Exists(rkApp.GetValue("StickyNotes").ToString()))
                        rkApp.SetValue("StickyNotes", Application.ExecutablePath.ToString());
                }
                catch
                {
                    rkApp.SetValue("StickyNotes", Application.ExecutablePath.ToString());
                }
            }
        }

        // aby nemohl být program spuštěn vícekrát vytvoří se mutex který kontroluje počet procesů dané aplikace a tato metoda pošle aplikaci zprávu "STICKY_MSG" která se porovná zda odpovídá zde nadefinované a pokud ano vykoná zadané akce
        internal class single_instance
        {
            public const int HWND_BROADCAST = 0xffff;
            public static readonly int WM_SHOWME = RegisterWindowMessage("STICKY_MSG");
            [DllImport("user32")]
            public static extern bool PostMessage(IntPtr hwnd, int msg, IntPtr wparam, IntPtr lparam);
            [DllImport("user32")]
            public static extern int RegisterWindowMessage(string message);

            public static void show_form()
            {
                single_instance.PostMessage(
                    (IntPtr)single_instance.HWND_BROADCAST,
                    single_instance.WM_SHOWME,
                    IntPtr.Zero,
                    IntPtr.Zero);
            }
        }
    }
}
